/// @file   IO.h
/// @brief  Filesystem IO.
/// @author Alejandro Álvarez Ayllon <aalvarez@cern.ch>
#ifndef IO_H
#define	IO_H

#include <dmlite/cpp/dmlite.h>
#include <dmlite/cpp/io.h>
#include <fstream>

namespace dmlite {

  class StdIOFactory: public IOFactory {
   public:
    StdIOFactory();
    ~StdIOFactory();

    void configure(const std::string& key, const std::string& value) throw (DmException);
    IODriver* createIODriver(PluginManager* pm) throw (DmException);

   private:
    std::string passwd_;
    bool        useIp_;
  };

  class StdIODriver: public IODriver {
   public:
    StdIODriver(std::string passwd, bool useIp);
    ~StdIODriver();

    std::string getImplId() const throw();

    void setStackInstance(StackInstance* si) throw (DmException);
    void setSecurityContext(const SecurityContext* ctx) throw (DmException);

    IOHandler* createIOHandler(const std::string& pfn, int openmode,
                              const Extensible& extras) throw (DmException);

    void doneWriting(const std::string& pfn,
                     const Extensible& params) throw (DmException);

   private:
    StackInstance*         si_;
    const SecurityContext* secCtx_;

    std::string passwd_;
    bool        useIp_;
  };

  class StdIOHandler: public IOHandler {
  public:
    StdIOHandler(const std::string& path,
                 int openmode) throw (DmException);
    ~StdIOHandler();
    
    void   close(void) throw (DmException);
    size_t read (char* buffer, size_t count) throw (DmException);
    size_t write(const char* buffer, size_t count) throw (DmException);
    void   seek (off_t offset, Whence whence) throw (DmException);
    off_t  tell (void) throw (DmException);
    void   flush(void) throw (DmException);
    bool   eof  (void) throw (DmException);

  protected:
    int  fd_;
    bool eof_;
    long pos_;
  };

};

#endif	// IO_H
